@@ -1,107 +1,175 @@
+import posixpath
 import select
+import stat
 import threading
 
+from interface_meta import override
+
+from omniduct.errors import DuctAuthenticationError
+from omniduct.filesystems.base import FileSystemFileDesc
 from omniduct.remotes.base import RemoteClient
 from omniduct.utils.debug import logger
+from omniduct.utils.processes import SubprocessResults
 
+# Python 2 compatibility imports
 try:
     import SocketServer
 except ImportError:
     import socketserver as SocketServer
 
+try:
+    FileNotFoundError
+except NameError:
+    FileNotFoundError = IOError
+
+__all__ = ['ParamikoSSHClient']
+
 
 class ParamikoSSHClient(RemoteClient):
+    """
+    An experimental SSH client that uses a `paramiko` rather than command-line
+    SSH backend. This client has been fully implemented and should work as is,
+    but until it receives further testing, we recommend using the cli backed SSH
+    client.
+    """
 
     PROTOCOLS = ['ssh_paramiko']
+    DEFAULT_PORT = 22
 
+    @override
     def _init(self):
         logger.warning("The Paramiko SSH client is still under development, \
                         and is not ready for use as a daily driver.")
 
+    @override
     def _connect(self):
         import paramiko  # Imported here due to relatively slow import
         self.__client = paramiko.SSHClient()
         self.__client.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())
         self.__client.load_system_host_keys()
 
-        self.__client.connect(self.host, username=self.username)
+        try:
+            self.__client.connect(self.host, username=self.username)
+            self.__client_sftp = paramiko.SFTPClient.from_transport(self.__client.get_transport())
+        except paramiko.SSHException as e:
+            if len(e.args) == 1 and e.args[0] == 'No authentication methods available':
+                raise DuctAuthenticationError(e.args[0])
+            raise e
 
+    @override
     def _is_connected(self):
         try:
             return self.__client.get_transport().is_active()
         except:
             return False
 
+    @override
     def _disconnect(self):
         try:
+            self.__client_sftp.close()
             return self.__client.close()
         except:
             pass
 
+    @override
     def _execute(self, cmd, **kwargs):
-        return self.__client.exec_command(cmd)
-
-    def _copy_to_local(self, source, dest):
-        raise NotImplementedError
-
-    def _copy_from_local(self, source, dest):
-        raise NotImplementedError
+        stdin, stdout, stderr = self.__client.exec_command(cmd)
+        returncode = stdout.channel.recv_exit_status()
+        return SubprocessResults(
+            returncode=returncode,
+            stdout=stdout.read(),
+            stderr=stderr.read()
+        )
 
+    @override
     def _port_forward_start(self, local_port, remote_host, remote_port):
         logger.debug('Now forwarding port {} to {}:{} ...'.format(local_port, remote_host, remote_port))
 
-        remote_host, remote_port = get_host_port(remote_host, remote_port)
-
         try:
             server = forward_tunnel(local_port, remote_host, remote_port, self.__client.get_transport())
         except KeyboardInterrupt:
             print('C-c: Port forwarding stopped.')
         return server
 
-    def _port_forward_stop(self, local_port, remote_host, remote_port, server):
-        server.shutdown()
+    @override
+    def _port_forward_stop(self, local_port, remote_host, remote_port, connection):
+        connection.shutdown()
 
+    @override
     def _is_port_bound(self, host, port):
         return True
 
-    # FileSystem methods
+    # Path properties and helpers
+    @override
+    def _path_home(self):
+        return self.execute('echo ~', skip_cwd=True).stdout.decode('utf-8').strip()
 
+    @override
+    def _path_separator(self):
+        return '/'
+
+    # File node properties
+    @override
     def _exists(self, path):
-        raise NotImplementedError
+        try:
+            self.__client_sftp.stat(path)
+            return True
+        except FileNotFoundError:
+            return False
 
+    @override
     def _isdir(self, path):
-        raise NotImplementedError
+        try:
+            return stat.S_ISDIR(self.__client_sftp.stat(path).st_mode)
+        except FileNotFoundError:
+            return False
 
+    @override
     def _isfile(self, path):
-        raise NotImplementedError
+        try:
+            return not stat.S_ISDIR(self.__client_sftp.stat(path).st_mode)
+        except FileNotFoundError:
+            return False
 
-    def _listdir(self, path):
-        raise NotImplementedError
+    # Directory handling and enumeration
+    @override
+    def _dir(self, path):
+        for attrs in self.__client_sftp.listdir_attr(path):
+            yield FileSystemFileDesc(
+                fs=self,
+                path=posixpath.join(path, attrs.filename),
+                name=attrs.filename,
+                type='directory' if stat.S_ISDIR(attrs.st_mode) else 'file',  # TODO: What about links, which are of form: lrwxrwxrwx?
+                bytes=attrs.st_size,
+                owner=attrs.st_uid,
+                group=attrs.st_gid,
+                last_modified=attrs.st_mtime,
+            )
 
-    def _showdir(self, path):
-        raise NotImplementedError
+    @override
+    def _mkdir(self, path, recursive, exist_ok):
+        if exist_ok and self.isdir(path):
+            return
+        assert self.execute('mkdir ' + ('-p ' if recursive else '') + '"{}"'.format(path)).returncode == 0, "Failed to create directory at: `{}`".format(path)
+
+    @override
+    def _remove(self, path, recursive):
+        assert self.execute('rm -f ' + ('-r ' if recursive else '') + '"{}"'.format(path)).returncode == 0, "Failed to remove file(s) at: `{}`".format(path)
 
     # File handling
-
-    def _file_read_(self, path, size=-1, offset=0, binary=False):
-        raise NotImplementedError
-
-    def _file_append_(self, path, s, binary):
-        raise NotImplementedError
-
-    def _file_write_(self, path, s, binary):
-        raise NotImplementedError
+    @override
+    def _open(self, path, mode):
+        """
+        Paramiko offers a complete file-like abstraction for files opened over
+        sftp, so we use that abstraction rather than a `FileSystemFile`. Results
+        should be indistinguishable.
+        """
+        return self.__client_sftp.open(path, mode=mode)
 
 
 # Port Forwarding Utility Code
 # Largely based on code from: https://github.com/paramiko/paramiko/blob/master/demos/forward.py
 
-SSH_PORT = 22
-DEFAULT_PORT = 4000
-
-g_verbose = True
-
-
 class ForwardServer (SocketServer.ThreadingTCPServer):
     daemon_threads = True
     allow_reuse_address = True
@@ -149,21 +217,14 @@ def forward_tunnel(local_port, remote_host, remote_port, transport):
     # this is a little convoluted, but lets me configure things for the Handler
     # object.  (SocketServer doesn't give Handlers any way to access the outer
     # server normally.)
-    class SubHander (Handler):
+    class SubHandler(Handler):
         chain_host = remote_host
         chain_port = remote_port
         ssh_transport = transport
-    server = ForwardServer(('', local_port), SubHander)
+    server = ForwardServer(('', local_port), SubHandler)
 
     t = threading.Thread(target=server.serve_forever)
     t.setDaemon(True)  # don't hang on exit
     t.start()
 
     return server
-
-
-def get_host_port(spec, default_port=22):
-    "parse 'hostname:22' into a host and port, with the port optional"
-    args = (spec.split(':', 1) + [default_port])[:2]
-    args[1] = int(args[1])
-    return args[0], args[1]
